#PBS -l nodes=1:ppn=16:xeon16
#PBS -l walltime=0:25:0

import os
import sys
from glob import glob
from optparse import OptionParser
import numpy as np
from ase.visualize import view
from ase.io import read, write
from gpaw import GPAW, PoissonSolver, FermiDirac, Davidson, PW
from gpaw.utilities import h2gpts
from gpaw.xc.libvdwxc import vdw_df, vdw_df2, vdw_df_cx
from gpaw.mpi import world

p = OptionParser()
p.add_option('--s22', action='store_true')
p.add_option('--ref', action='store_true')

opts, args = p.parse_args()
if not opts.s22 and not opts.ref:
    calc_all = True

calc_s22 = opts.s22 or calc_all
calc_ref = opts.ref or calc_all

cwd = os.getcwd()
path0, mydirname = os.path.split(cwd)
molno = 1#int(mydirname.split('.')[1])
fnames = glob('../xyz_displaced/s%02d_??.xyz' % molno)
fnames.sort()

configs = [read(fname) for fname in fnames]

atoms = configs[-1]
atoms.pbc = 1
tmp = atoms.copy()
tmp.center(vacuum=2.0)
atoms.cell = tmp.cell # Atoms remain in corner (lower egg-box)

d1 = configs[1].positions[-1]
d2 = configs[2].positions[-1]
dx1 = d2 - d1
DX = configs[2].positions - configs[1].positions

i1 = [i for i, pos in enumerate(DX) if abs(np.vdot(dx1, DX[i])) > 1e-4]
i2 = [i for i, pos in enumerate(DX) if abs(np.vdot(dx1, DX[i])) < 1e-4]

mol1 = atoms[i1]
mol2 = atoms[i2]
assert len(mol1) + len(mol2) == len(atoms)

def getcalc(**kwargs):
    kwargs1 = dict(#gpts=h2gpts(0.17, atoms.cell, idiv=8),
                   #mode=PW(400),
                   gpts=(64, 24, 28),
                   eigensolver=Davidson(4),#'rmm-diis',
                   #parallel=dict(augment_grids=True),
                   #basis='dzp',
                   convergence=dict(density=1e-5),
                   occupations=FermiDirac(0.0),
                   xc='PBE',
                   #poissonsolver=PoissonSolver(relax='GS', eps=2e-12)
                   )
    kwargs1.update(kwargs)
    calc = GPAW(**kwargs1)
    return calc


def lock_try(lockfname):
    decision = np.zeros(1, dtype=int)
    fd = None
    if world.rank == 0:
        if not os.path.exists(lockfname):
            decision[0] = 1
            fd = open(lockfname, 'w')
    world.sum(decision)
    decision = decision[0]
    if decision and world.rank == 0:
        print 'calculate %s' % lockfname
    return fd, decision


if calc_s22:
    atoms.calc = getcalc(txt='gpaw.s22.txt')
if calc_ref:
    mol1.calc = getcalc(txt='gpaw.mol1.txt')
    mol2.calc = getcalc(txt='gpaw.mol2.txt')

ncalc = 0
for i, config in enumerate(configs):
    atoms.positions[:] = config.positions
    id = '%03d' % i
    #if ncalc > 0:
    #    break

    lock_s22 = 'results.s22.%s.txt' % id
    lock_mol1 = 'results.mol1.%s.txt' % id
    lock_mol2 = 'results.mol2.txt'  # stationary

    mol1.positions[:] = atoms.positions[i1]
    mol2.positions[:] = atoms.positions[i2]

    x = np.linalg.norm(mol2.get_center_of_mass() - mol1.get_center_of_mass())

    if calc_s22:
        fd, decision = lock_try(lock_s22)
        if decision:
            ncalc += 1
            atoms.calc.set(txt='gpaw.s22.%s.txt' % id)
            E = atoms.get_potential_energy()
            F0 = atoms.get_forces()
            write('out.s22.%s.traj' % id, atoms)
            if fd is not None:
                fd.write('%15.10f %15.10f\n' % (x, E))
                fd.close()

    if calc_ref:
        fd, decision = lock_try(lock_mol2)
        if decision:
            ncalc += 1
            Eref2 = mol2.get_potential_energy()
            Fref2 = mol2.get_forces()
            write('out.mol2.traj', mol2)
            if fd is not None:
                fd.write('%15.10f %15.10f\n' % (x, Eref2))
                fd.close()
            mol2.calc = None


    if calc_ref:
        fd, decision = lock_try(lock_mol1)
        if decision:
            ncalc += 1
            mol1.calc.set(txt='gpaw.mol1.%s.txt' % id)
            Eref1 = mol1.get_potential_energy()
            Fref1 = mol1.get_forces()
            write('out.mol1.%s.traj' % id, mol1)
            if fd is not None:
                fd.write('%15.10f %15.10f\n' % (x, Eref1))
                fd.close()
