import os
import sys
from glob import glob
import numpy as np
import pylab as pl

from optparse import OptionParser
p = OptionParser()
p.add_option('--noref', action='store_true',
             help='do not subtract reference energies')
opts, dirnames = p.parse_args()

fig = pl.figure()
ax = fig.add_subplot(111)

for dirname in dirnames:
    s22fnames = glob('%s/results.s22.*.txt' % dirname)
    s22fnames.sort()

    def load(name):
        try:
            fd = open(os.path.join(dirname, name))
        except IOError:
            raise KeyError('missing file: %s' % name)

        values = map(float, fd.read().split())
        try:
            x, e = values
        except ValueError:
            raise ValueError('incomplete or corrupted file: %s' % name)

        return x, e

    try:
        _, Eref2 = load('results.mol2.txt')
    except (ValueError, KeyError) as err:
        print err
        continue

    xval = []
    Eval = []
    Eref1val = []

    for fname in s22fnames:
        _, fname = os.path.split(fname)
        tokens = fname.split('.')
        configno = int(tokens[-2])

        try:
            _, Eref1 = load('results.mol1.%03d.txt' % configno)
        except (ValueError, KeyError) as err:
            print err
            continue

        try:
            x, Es22 = load(fname)
        except ValueError as err:
            print err
            continue

        Eref1val.append(Eref1)
        xval.append(x)
        Eval.append(Es22)

    xval = np.array(xval)
    Eval = np.array(Eval)
    Eref1val = np.array(Eref1val)

    if opts.noref:
        E = Eval
        #E -= np.min(E)
    else:
        E = Eval - Eref1val - Eref2
    #pl.plot(xval, Eref1val, 'o-')
    if 'pw' in dirname:
        style = 's--'
    else:
        style = 'o-'
    ax.plot(xval, E, style, label=dirname)

    #pl.plot(xval, Eval - 2 * Eref2, 'o--', label=dirname)

    ax.axhline(0.0, ls=':', color='k')
ax.legend(loc='best')
pl.show()
